searchNodes=[{"ref":"Mockery.html","title":"Mockery","type":"module","doc":"Core functionality"},{"ref":"Mockery.html#mock/3","title":"Mockery.mock/3","type":"function","doc":"Function used to create mock in context of single test process. Mock created in test won&#39;t leak to another process (other test, spawned Task, GenServer...). It can be used safely in asynchronous tests. Mocks can be created with static value: mock Mod, [fun: 2], &quot;mocked value&quot; or function: mock Mod, [fun: 2], fn(_, arg2) -&gt; arg2 end Keep in mind that function inside mock must have same arity as original one. This: mock Mod, [fun: 2], &amp;to_string/1 will raise an error. It is also possible to mock function with given name and any arity mock Mod, :fun, &quot;mocked value&quot; but this version doesn&#39;t support function as value. Also, multiple mocks for same module are chainable Mod |&gt; mock(:fun1, &quot;value&quot;) |&gt; mock([fun2: 1], &amp;string/1)"},{"ref":"Mockery.html#new/2","title":"Mockery.new/2","type":"function","doc":""},{"ref":"Mockery.html#of/2","title":"Mockery.of/2","type":"function","doc":"Function used to prepare module for mocking. For Mix.env other than :test it returns module given in the first argument. If Mix.env equal :test it creates a proxy to the original module. When Mix is missing it assumes that env is :prod Examples Prepare for mocking (elixir module) defmodule Foo do @bar Mockery.of(&quot;Bar&quot;) def foo do @bar.bar() end end It is also possible to pass the module in elixir format @bar Mockery.of(Bar) but it is not recommended as it creates an unnecessary compile-time dependency (see mix xref graph output for both versions). Prepare for mocking (erlang module) defmodule Foo do @crypto Mockery.of(:crypto) def foo do @crypto.rand_seed() end end Prepare for mocking with global mock # test/support/global_mocks/bar.ex defmodule BarGlobalMock do def bar, do: :mocked end # lib/foo.ex defmodule Foo do @bar Mockery.of(Bar, by: BarGlobalMock) def foo do @bar.bar() end end OTP21+ Internally mockery is using tuple calls to pass additional data to its proxy module when mock is called. Tuple calls are disabled by default in OTP21+ and require additional compile flag to be reenabled. defmodule Foo do @compile :tuple_calls @bar Mockery.of(&quot;Bar&quot;) # ... end If you don&#39;t want to reenable tuple calls, there&#39;s also new macro-based alternative (for more information see Mockery.Macro module)."},{"ref":"Mockery.html#t:elixir_module_as_string/0","title":"Mockery.elixir_module_as_string/0","type":"type","doc":"Used to avoid unnecessary compile-time dependencies between modules Examples defmodule Foo do # this creates compile-time dependency between Foo and Bar @bar1 Mockery.of(Bar) # same result but without compile-time dependency @bar2 Mockery.of(&quot;Bar&quot;) end mix xref graph can be used to check difference between module and string versions"},{"ref":"Mockery.html#t:proxy_tuple/0","title":"Mockery.proxy_tuple/0","type":"opaque","doc":"Mockery uses tuple calls to send additional data to internal proxy module"},{"ref":"Mockery.Assertions.html","title":"Mockery.Assertions","type":"module","doc":"Set of additional assertion functions. NOTE: Mockery doesn&#39;t keep track of function calls: on modules that weren&#39;t prepared by Mockery.of/2 or Mockery.Macro.mockable/2 for MIX_ENV other than :test for function called outside of test process (in spawned Task, GenServer etc.)"},{"ref":"Mockery.Assertions.html#assert_called/2","title":"Mockery.Assertions.assert_called/2","type":"function","doc":"Asserts that function from given module with given name or name and arity was called at least once. Examples Assert Mod.fun/2 was called assert_called Mod, fun: 2 Assert any function named :fun from module Mod was called assert_called Mod, :fun"},{"ref":"Mockery.Assertions.html#assert_called/3","title":"Mockery.Assertions.assert_called/3","type":"macro","doc":"Asserts that function from given module with given name was called at least once with arguments matching given pattern. Examples Assert Mod.fun/2 was called with given args list assert_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;] You can also use unbound variables inside args pattern assert_called Mod, :fun, [&quot;a&quot;, _second]"},{"ref":"Mockery.Assertions.html#assert_called/4","title":"Mockery.Assertions.assert_called/4","type":"macro","doc":"Asserts that function from given module with given name was called given number of times with arguments matching given pattern. Similar to assert_called/3 but instead of checking if function was called at least once, it checks if function was called specific number of times. Examples Assert Mod.fun/2 was called with given args 5 times assert_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], 5 Assert Mod.fun/2 was called with given args from 3 to 5 times assert_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], 3..5 Assert Mod.fun/2 was called with given args 3 or 5 times assert_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], [3, 5]"},{"ref":"Mockery.Assertions.html#refute_called/2","title":"Mockery.Assertions.refute_called/2","type":"function","doc":"Asserts that function from given module with given name or name and arity was NOT called. Examples Assert Mod.fun/2 wasn&#39;t called refute_called Mod, fun: 2 Assert any function named :fun from module Mod wasn&#39;t called refute_called Mod, :fun"},{"ref":"Mockery.Assertions.html#refute_called/3","title":"Mockery.Assertions.refute_called/3","type":"macro","doc":"Asserts that function from given module with given name was NOT called with arguments matching given pattern. Examples Assert Mod.fun/2 wasn&#39;t called with given args list refute_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;] You can also use unbound variables inside args pattern refute_called Mod, :fun, [&quot;a&quot;, _second]"},{"ref":"Mockery.Assertions.html#refute_called/4","title":"Mockery.Assertions.refute_called/4","type":"macro","doc":"Asserts that function from given module with given name was NOT called given number of times with arguments matching given pattern. Similar to refute_called/3 but instead of checking if function was called at least once, it checks if function was called specific number of times. Examples Assert Mod.fun/2 was not called with given args 5 times refute_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], 5 Assert Mod.fun/2 was not called with given args from 3 to 5 times refute_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], 3..5 Assert Mod.fun/2 was not called with given args 3 or 5 times refute_called Mod, :fun, [&quot;a&quot;, &quot;b&quot;], [3, 5]"},{"ref":"Mockery.Error.html","title":"Mockery.Error","type":"exception","doc":""},{"ref":"Mockery.History.html","title":"Mockery.History","type":"module","doc":"Provides calls history for Mockery.Assertions macros. It&#39;s disabled by default. It can be enabled/disabled globally by following config config :mockery, history: true Or for single test process Mockery.History.enable_history() Mockery.History.disable_history() Process config has higher priority than global config"},{"ref":"Mockery.History.html#disable_history/0","title":"Mockery.History.disable_history/0","type":"function","doc":"Disables history in scope of single test process use Mockery test &quot;example&quot; do #... disable_history() assert_called Foo, :bar, [_, :a] end"},{"ref":"Mockery.History.html#enable_history/0","title":"Mockery.History.enable_history/0","type":"function","doc":"Enables history in scope of single test process use Mockery test &quot;example&quot; do #... enable_history() assert_called Foo, :bar, [_, :a] end"},{"ref":"Mockery.History.html#enable_history/1","title":"Mockery.History.enable_history/1","type":"function","doc":""},{"ref":"Mockery.Macro.html","title":"Mockery.Macro","type":"module","doc":"Alternative macro-based way to prepare module for mocking/asserting."},{"ref":"Mockery.Macro.html#mockable/2","title":"Mockery.Macro.mockable/2","type":"macro","doc":"Function used to prepare module for mocking/asserting. For Mix.env other than :test it returns the first argument unchanged. If Mix.env equal :test it creates a proxy to the original module. When Mix is missing it assumes that env is :prod. Examples Prepare for mocking defmodule Foo do import Mockery.Macro def foo do mockable(Bar).bar() end end Prepare for mocking with global mock # test/support/global_mocks/bar.ex defmodule BarGlobalMock do def bar, do: :mocked end # lib/foo.ex defmodule Foo do import Mockery.Macro def foo do mockable(Bar, by: BarGlobalMock).bar() end end Mockery.of/2 comparison It&#39;s based on macro and process dictionary instead of on tuple calls. (Tuple calls are disabled by default in OTP21+ and require additional compile flag to be reenabled) It doesn&#39;t support passing module names as a string as it don&#39;t create unwanted compile-time dependencies between modules Potential issues Output of mockable/2 macro should not be bind to variable or module attribute. If it happens, you&#39;ll see a compilation warning at best, and in the worst case Mockery won&#39;t work correctly."},{"ref":"readme.html","title":"Mockery","type":"extras","doc":"Mockery Simple mocking library for asynchronous testing in Elixir. Readme and documentation for last stable version are available on hex"},{"ref":"readme.html#advantages","title":"Mockery - Advantages","type":"extras","doc":"Mockery does not override your modules Mockery does not replace modules by aliasing Mockery does not require to pass modules as function parameter Mockery does not require to create callbacks or wrappers around libraries Mockery does not create modules during runtime (neither by defmodule/2 nor Module.create/3) Mockery does not allow to mock non-existent function Mockery does not share any data between test processes"},{"ref":"readme.html#disadvantages","title":"Mockery - Disadvantages","type":"extras","doc":"Mockery is not designed for libraries as it would force end user to download Mockery as dependency of dependency Mockery can cause issues with dialyzer if you are using dialyzer with MIX_ENV=test"},{"ref":"readme.html#upgrading-erlang-otp-to-21","title":"Mockery - Upgrading Erlang/OTP to 21","type":"extras","doc":"Erlang/OTP 21 contains some changes that prevent the package from functioning as before. Please check migration guide."},{"ref":"readme.html#getting-started","title":"Mockery - Getting started","type":"extras","doc":"Installation def deps do [ {:mockery, &quot;~&gt; 2.3.0&quot;, runtime: false} ] end Preparation of the module for mocking # lib/my_app/foo.ex defmodule MyApp.Foo do import Mockery.Macro alias MyApp.Bar def baz, do: mockable(Bar).function() end"},{"ref":"readme.html#basic-usage","title":"Mockery - Basic usage","type":"extras","doc":"Static value mock defmodule MyApp.Controller do # ... import Mockery.Macro def all do mockable(MyApp.UserService).users() end def filtered do mockable(MyApp.UserService).users(&quot;filter&quot;) end end # tests defmodule MyApp.ControllerTest do # ... import Mockery test &quot;mock any function :users from MyApp.UserService&quot; do mock MyApp.UserService, :users, &quot;mock&quot; assert all() == &quot;mock&quot; assert filtered() == &quot;mock&quot; end test &quot;mock MyApp.UserService.users/0&quot; do mock MyApp.UserService, [users: 0], &quot;mock&quot; assert all() == &quot;mock&quot; refute filtered() == &quot;mock&quot; end test &quot;mock MyApp.UserService.users/0 with default value&quot; do mock MyApp.UserService, users: 0 assert all() == :mocked refute filtered() == :mocked end test &quot;chaining multiple mocks for same module&quot; do UserService |&gt; mock([users: 0], &quot;mock value&quot;) |&gt; mock([users: 1], &quot;mock value&quot;) # ... end end Dynamic mock Instead of using a static value, you can use a function with the same arity as original one. defmodule Foo do def bar(value), do: value end # prepare tested module defmodule Other do import Mockery.Macro def parse(value) do mockable(Foo).bar(value) end end # tests defmodule OtherTest do # ... import Mockery test &quot;with dynamic mock&quot; do mock Foo, [bar: 1], fn(value)-&gt; String.upcase(value) end assert parse(&quot;test&quot;) == &quot;TEST&quot; end end"},{"ref":"readme.html#checking-if-function-was-called","title":"Mockery - Checking if function was called","type":"extras","doc":"# prepare tested module defmodule Tested do import Mockery.Macro def call(value, opts) do mockable(Foo).bar(value) end end # tests defmodule TestedTest do # ... import Mockery.Assertions # use Mockery # when you need to import both Mockery and Mockery.Assertions test &quot;assert any function bar from module Foo was called&quot; do Tested.call(1, %{}) assert_called Foo, :bar end test &quot;assert Foo.bar/2 was called&quot; do Tested.call(1, %{}) assert_called Foo, bar: 2 end test &quot;assert Foo.bar/2 was called with given args&quot; do Tested.call(1, %{}) assert_called Foo, :bar, [1, %{}] end test &quot;assert Foo.bar/1 was called with given arg (using variable)&quot; do params = %{a: 1, b: 2} Tested.call(params) assert_called Foo, :bar, [^params] # we need to use pinning here since assert_called/3 is a macro # and not a regular function call and it gets expanded accordingly end test &quot;assert Foo.bar/2 was called with 1 as first arg&quot; do Tested.call(1, %{}) assert_called Foo, :bar, [1, _] end test &quot;assert Foo.bar/2 was called with 1 as first arg 5 times&quot; do # ... assert_called Foo, :bar, [1, _], 5 end test &quot;assert Foo.bar/2 was called with 1 as first arg from 3 to 5 times&quot; do # ... assert_called Foo, :bar, [1, _], 3..5 end test &quot;assert Foo.bar/2 was called with 1 as first arg 3 or 5 times&quot; do # ... assert_called Foo, :bar, [1, _], [3, 5] end end Refute Every assert_called/x function/macro has its refute_called/x counterpart. For more information see docs History Mockery.History module provides more descriptive failure messages for assert_called/{3,4} and refute_called/{3,4} that includes a colorized list of arguments passed to a given function in the scope of a single test process. Disabled by default. For more information see docs"},{"ref":"readme.html#global-mock","title":"Mockery - Global mock","type":"extras","doc":"Useful when you need to use the same mock many times across different tests defmodule Foo do def bar, do: 1 def baz, do: 2 end defmodule FooGlobalMock do def bar, do: :mocked end # prepare tested module defmodule Other do import Mockery.Macro def bar, do: mockable(Foo, by: FooGlobalMock).bar() def baz, do: mockable(Foo, by: FooGlobalMock).baz() end # tests defmodule OtherTest do # ... test &quot;with global mock&quot; do assert Other.bar == :mocked assert Other.baz == 2 end end Restrictions Global mock module doesn&#39;t have to contain every function exported by the original module, but it cannot contain a function which is not exported by the original module. It means that: when you remove a function from the original module, you have to remove it from global mock module or Mockery will raise exception when you change a function name in the original module, you have to change it in global mock module or Mockery will raise exception when you change a function arity in the original module, you have to change it in global mock module or Mockery will raise exception"},{"ref":"readme.html#advanced-examples","title":"Mockery - Advanced examples","type":"extras","doc":"For advanced usage examples see EXAMPLES.md"},{"ref":"readme.html#external-resources","title":"Mockery - External resources","type":"extras","doc":"https://stephenbussey.com/2018/02/15/my-favorite-elixir-testing-tool-mockery"},{"ref":"readme.html#license","title":"Mockery - License","type":"extras","doc":"Copyright 2017-2019 Tobiasz Małecki tobiasz.malecki@appunite.com Licensed under the Apache License, Version 2.0 (the &quot;License&quot;); you may not use this file except in compliance with the License. You may obtain a copy of the License at http://www.apache.org/licenses/LICENSE-2.0 Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an &quot;AS IS&quot; BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License."},{"ref":"examples.html","title":"Examples","type":"extras","doc":"Examples"},{"ref":"examples.html#situations-triggered-by-another-process","title":"Examples - Situations triggered by another process","type":"extras","doc":"Let&#39;s assume we have a unique resource and function that is fetching it if exist or create new otherwise. @service Mockery.of(&quot;Service&quot;) def fetch_or_create do with \\ {:error, :not_found} &lt;- @service.fetch(), {:ok, resource} &lt;- @service.create() do {:ok, resource} else {:ok, resource} -&gt; {:ok, resource} {:error, %Changeset{errors: [identifier: &quot;has already been taken&quot;]}} -&gt; fetch_or_create() end end {:error, %Changeset{}} in else clause can match only if between @service.fetch and @service.create there was another process that successfully created resource (Service.create()) How to cover {:error, %Changeset{}} -&gt; fetch_or_create() with tests? mock Service, :create, {:error, %Ecto.Changeset{...}} mock Service, [fetch: 0], fn -&gt; mock Service, [fetch: 0], &amp;Service.fetch/0 {:error, :not_found} end assert {:ok, %Resource{}} = fetch_or_create() In solution above: Service.create/0 is permanently mocked to return {:error, %Changeset{}} First call of Service.fetch/0 changes mock for next calls to original function and returns {:error, :not_found} Second call of Service.fetch/0 returns {:ok, resource}"},{"ref":"examples.html#task-supervisor","title":"Examples - Task.Supervisor","type":"extras","doc":"Run tasks synchronously in test environment for easier testing defmodule MyApp.Service do def something, do: &quot;value&quot; end defmodule MyApp.Controller do @task_supervisor Mockery.of(&quot;Task.Supervisor&quot;) @service Mockery.of(&quot;MyApp.Service&quot;) def action do @task_supervisor.start_child(MyApp.TaskSupervisor, fn-&gt; @service.something() end) end end defmodule MyApp.ControllerTest do use ExUnit.Case, async: true import Mockery.Assertions test &quot;something is called&quot; do mock Task.Supervisor, :start_child, fn(_, fun) -&gt; fun.() end MyApp.Controller.action() assert_called MyApp.Service, :something end end"},{"ref":"changelog.html","title":"Changelog","type":"extras","doc":"Changelog"},{"ref":"changelog.html#2-3-1-2019-10-10","title":"Changelog - 2.3.1 (2019-10-10)","type":"extras","doc":"Fixed issue preventing module attributes to be used in assert_called macros #34"},{"ref":"changelog.html#2-3-0-2018-10-22","title":"Changelog - 2.3.0 (2018-10-22)","type":"extras","doc":"Allowed multiple mocks when using pipe operator or nested calls #27 Deprecated Mockery.History.enable_history/1 in favor of Mockery.History.enable_history/0 and Mockery.History.disable_history/0"},{"ref":"changelog.html#2-2-0-2018-06-17","title":"Changelog - 2.2.0 (2018-06-17)","type":"extras","doc":"Added Mockery.Macro.mockable/2 macro as alternative way to prepare module for mocking Deprecated Mockery.new/2"},{"ref":"changelog.html#2-1-0-2018-02-19","title":"Changelog - 2.1.0 (2018-02-19)","type":"extras","doc":"Added Mockery.new/2 Fixed typo in error message #21 Removed support for Elixir 1.1 and 1.2"},{"ref":"changelog.html#2-0-2-2017-10-25","title":"Changelog - 2.0.2 (2017-10-25)","type":"extras","doc":"Fixed Mockery.of/2 typespecs"},{"ref":"changelog.html#2-0-1-2017-10-13","title":"Changelog - 2.0.1 (2017-10-13)","type":"extras","doc":"When Mix is missing, Mockery.of/2 assumes that env is :prod"},{"ref":"changelog.html#2-0-0-2017-10-08","title":"Changelog - 2.0.0 (2017-10-08)","type":"extras","doc":"Removed Mockery.Heritage Global mocks will be handled without macros by pure elixir modules. Some new restrictions have been added for global mock modules (see global mock section in README) Changed Mockery.mock/3 when value is function (dynamic mock) [function_name: arity] syntax remains unchanged :function_name syntax will raise error Changed Mockery.of/2 output run mix compile --force when upgrading from Mockery 1.x.x"},{"ref":"changelog.html#1-4-0-2017-10-02","title":"Changelog - 1.4.0 (2017-10-02)","type":"extras","doc":"Mockery.mock/2 and Mockery.mock/3 are now chainable"},{"ref":"changelog.html#1-3-1-2017-09-21","title":"Changelog - 1.3.1 (2017-09-21)","type":"extras","doc":"Minor format fix in Mockery.History failure messages"},{"ref":"changelog.html#1-3-0-2017-09-15","title":"Changelog - 1.3.0 (2017-09-15)","type":"extras","doc":"Added string version for Mockery.of/2 (now recommended) Fixed mocking of erlang modules"},{"ref":"changelog.html#1-2-0-2017-09-13","title":"Changelog - 1.2.0 (2017-09-13)","type":"extras","doc":"Introduced Mockery.History"},{"ref":"changelog.html#1-1-1-2017-09-05","title":"Changelog - 1.1.1 (2017-09-05)","type":"extras","doc":"More descriptive errors (when args are not list) for: Mockery.Assertions.assert_called/3 Mockery.Assertions.refute_called/3 Mockery.Assertions.assert_called/4 Mockery.Assertions.refute_called/4 Removed unnecessary Elixir. namespace from module names in error messages"},{"ref":"changelog.html#1-1-0-2017-08-07","title":"Changelog - 1.1.0 (2017-08-07)","type":"extras","doc":"Added Mockery.mock/2 (Mockery.mock/3 value defaults to :mocked) Added Mockery.Assertions.assert_called/4 macro Added Mockery.Assertions.refute_called/4 macro"},{"ref":"changelog.html#1-0-1-2017-08-02","title":"Changelog - 1.0.1 (2017-08-02)","type":"extras","doc":"Fixed issue when mocking with nil or false"},{"ref":"changelog.html#1-0-0-2017-07-30","title":"Changelog - 1.0.0 (2017-07-30)","type":"extras","doc":"Added use Mockery for importing both Mockery and Mockery.Assertions Added Mockery.Assertions.refute_called/2 and Mockery.Assertions.refute_called/3 Loosen elixir version requirement from ~&gt; 1.3 to ~&gt; 1.1"},{"ref":"migration_to_otp21.html","title":"Migration to Erlang/OTP21","type":"extras","doc":"Migration to Erlang/OTP21"},{"ref":"migration_to_otp21.html#tuple-calls","title":"Migration to Erlang/OTP21 - Tuple calls","type":"extras","doc":"Due to this PR https://github.com/erlang/otp/pull/1499, it is now necessary to add @compile :tuple_calls in every module where old Mockery API was used. So, for example, most efficient way to make this defmodule MyProject do @foo Mockery.of(&quot;Foo&quot;) def bar, do: @foo.bar() def baz, do: @foo.baz() end works as expected after OTP upgrade is defmodule TupleCalls do defmacro __using__(_opts) do if function_exported?(Mix, :env, 0) &amp;&amp; Mix.env() == :test do quote do: @compile(:tuple_calls) end end end defmodule MyProject do use TupleCalls @foo Mockery.of(&quot;Foo&quot;) def bar, do: @foo.bar() def baz, do: @foo.baz() end This reenables tuple calls only for :test environment."},{"ref":"migration_to_otp21.html#macro-based-alternative","title":"Migration to Erlang/OTP21 - Macro-based alternative","type":"extras","doc":"For those who prefer not to reenable tuple calls, there is a new macro-based API. Previous example rewritten to use macros: defmodule MyProject do import Mockery.Macro def bar, do: mockable(Foo).bar() def baz, do: mockable(Foo).baz() end"}]